@@ -2,11 +2,11 @@ package Lock::Server;
 
 =head1 NAME
 
-    Lock::Server - Light-weight socket based resource locking manager.
+    Lock::Server - Light-weight RESTful socket based resource locking manager.
 
 =head1 DESCRIPTION
 
-    This creates a child process socket server that takes lock and 
+    This creates a socket server that handles lock and 
     unlock requests. The lock requests only return once a lock is
     obtained or a timeout has occurred. A lock may only be locked
     for a specific amount of time before the lock is timed out.
@@ -50,9 +50,10 @@ package Lock::Server;
         print "Lock server started in child thread $childPid\n";
     }
 
-    my $lockClient_A = $lockServer->client( "CLIENT_A" );
+    my $optional_args = { reconnect_attempts => 3, time_between_attempts => 1 };
+    my $lockClient_A = $lockServer->client( "CLIENT_A", $optional_args );
     my $lockClient_B = 
-        new Lock::Server::Client( "CLIENT_B", 'localhost', 888 );
+        new Lock::Server::Client( "CLIENT_B", 'localhost', 888, $optional_args );
 
     if( $lockClient_A->lock( "KEYA" ) ) {
        print "Lock Successfull for locker A and KEYA\n";
@@ -79,7 +80,7 @@ package Lock::Server;
     }
 
 =head1 METHODS
-    
+
 =cut
 
 use strict;
@@ -92,7 +93,7 @@ use IO::Socket::INET;
 
 use vars qw($VERSION);
 
-$VERSION = '1.1';
+$VERSION = '1.4';
 
 
 $Lock::Server::DEBUG = 0;
@@ -120,6 +121,9 @@ sub new {
         _id2pid              => {},
         _locks               => {},
         _locker_counts       => {},
+        attempts => $args->{reconnect_attemps} || 3,
+        time_between_attempts => $args->{time_between_attempts} || 3,
+
     }, $class;
 } #new
 
@@ -130,8 +134,8 @@ sub new {
 
 =cut
 sub client {
-    my( $self, $name ) = @_;
-    Lock::Server::Client->new( $name, $self->{host}, $self->{port} );
+    my( $self, $name, $args ) = @_;
+    Lock::Server::Client->new( $name, $self->{host}, $self->{port}, $args );
 }
 
 =head2 stop
@@ -141,8 +145,15 @@ sub client {
 =cut
 sub stop {
     my $self = shift;
+
+    for my $pid (keys %{$self->{_id2pid}}) {
+        _log( "Killing child proc $pid" );
+        kill 'INT', $pid;
+    }
+    
     if( my $pid = $self->{server_pid} ) {
         $self->{error} = "Sending INT signal to lock server of pid '$pid'";
+        _log( "Killing lock server proc $pid" );
         kill 'INT', $pid;
         return 1;
     }
@@ -159,26 +170,15 @@ sub stop {
 =cut
 sub start {
     my $self = shift;
-    my $listener_socket = new IO::Socket::INET(
-        Listen    => 10,
-        LocalAddr => "$self->{host}:$self->{port}",
-        );
-    unless( $listener_socket ) {
-        $self->{error} = "Unable to open socket on port '$self->{port}' : $! $@\n";
-        _log( "unable to start lock server : $@ $!." );
-        return 0;
-    }
-    binmode( $listener_socket, ':utf8' );
-    $listener_socket->autoflush;
     if( my $pid = fork ) {
         # parent
         $self->{server_pid} = $pid;
         return $pid;
     }
-    
-
+#    use Devel::SimpleProfiler;Devel::SimpleProfiler::start;
+    $0 = "LockServer";
     # child process
-    $self->run( $listener_socket );
+    $self->run;
 
 } #start
 
@@ -188,36 +188,40 @@ sub start {
 
 =cut
 sub run {
-    my( $self, $listener_socket ) = @_;
+    my $self = shift;
 
 
-    unless( $listener_socket ) { 
+    my( $listener_socket, $count );
+
+    my $st = time;
+    
+    until( $listener_socket || $count++ > $self->{attempts} ) {
         $listener_socket = new IO::Socket::INET(
             Listen    => 10,
             LocalAddr => "$self->{host}:$self->{port}",
             );
-        unless( $listener_socket ) {
-            $self->{error} = "Unable to open socket on port '$self->{port}' : $! $@\n";
-            _log( "unable to start lock server : $@ $!." );
-            return 0;
-        }
+        sleep $self->{time_between_attempts}*$count unless $listener_socket;
+    }
+    unless( $listener_socket ) {
+
+        $self->{error} = "Unable to open socket on port '$self->{port}' : $! $@\n";
+        _log( "unable to start lock server : $@ $!." );
+        return 0;
     }
 
     # if this is cancelled, make sure all child procs are killed too
     $SIG{INT} = sub {
         _log( "lock server : got INT signal. Shutting down." );
         $listener_socket && $listener_socket->close;
-        for my $pid (keys %{ $self->{_pids} } ) {
-            kill 'HUP', $pid;
-        }
+        kill 'HUP', keys %{ $self->{_pids} };
         exit;
     };
 
 
     while( my $connection = $listener_socket->accept ) {
-        _log( "lock server : incoming request" );
         my $req = <$connection>; 
         chomp $req;
+        _log( "lock server : incoming request : '$req'" );
 
         # could have headers, but ignore those. Find \n\n
         while( my $data = <$connection> ) {
@@ -225,8 +229,6 @@ sub run {
             last unless $data =~ /\S/;
         }
 
-        _log( "lock server : got request '$req'" );
-
         my( $cmd, $key, $locker_id ) = ( $req =~ m!^GET /([^/]+)/([^/]+)/?([^/]+)?! );
 
         if( $cmd eq 'CHECK' ) {
@@ -247,7 +249,6 @@ sub run {
 
 sub _check {
     my( $self, $connection, $key_to_check ) = @_;
-
     _log( "locker server check for key '$key_to_check'" );
 
     $self->{_locks}{$key_to_check} ||= [];
@@ -277,19 +278,19 @@ sub _check {
 
 sub _log {
     my $msg = shift;
-    print STDERR "$msg\n" if $Lock::Server::DEBUG;
+    print STDERR "Lock::Server : $msg\n" if $Lock::Server::DEBUG;
 }
 
 sub _lock {
     my( $self, $connection, $key_to_lock, $locker_id ) = @_;
 
-    _log( "lock server : lock request for '$locker_id' and key '$key_to_lock'" );
+    _log( "lock request :  for '$locker_id' and key '$key_to_lock'" );
 
     $self->{_locks}{$key_to_lock} ||= [];
     my $lockers = $self->{_locks}{$key_to_lock};
-
     #check for timed out lockers
     my $t = time;
+
     while( @$lockers && $t > $self->{_locker_counts}{$lockers->[0]}{$key_to_lock} ) {
         _log( "lock '$key_to_lock' timed out for locker '$lockers->[0]'" );
         if( 1 == keys %{ $self->{_locker_counts}{$lockers->[0]} } ) {
@@ -300,7 +301,6 @@ sub _lock {
         shift @$lockers;
     }
 
-
     if( 0 < (grep { $_ eq $locker_id } @$lockers) ) {
         _log( "lock request error. '$locker_id' already in the lock queue" );
         print $connection "0\n";
@@ -320,17 +320,17 @@ sub _lock {
             _log( "lock request : parent process associating '$locker_id' with pid '$pid' ".scalar(@$lockers)." lockers" );
             # parent
         } else {
-            # child
-            $SIG{HUP} = sub {
+#            use Devel::SimpleProfiler;Devel::SimpleProfiler::start;
+            $0 = "LockServer processing request";
+            $SIG{HUP} = $SIG{INT} = sub {
                 _log( "lock request : child $$ got HUP, so is now locked." );
-#                print $connection "$timeout_time\n";
- $connection->print( "$timeout_time\n" );
+                $connection->print( "$timeout_time\n" );
                 $connection->close;
-                undef $connection;
                 exit;
             };
             _log( "lock request : child $$ ready to wait" );
             sleep $self->{lock_attempt_timeout};
+            _log( "lock request failed : child $$ timed out" );
             print $connection "0\n";
             $connection->close;
             exit;
@@ -344,7 +344,7 @@ sub _lock {
 
 sub _unlock {
     my( $self, $connection, $key_to_unlock, $locker_id ) = @_;
-    _log( "lock server unlock for key '$key_to_unlock' for locker '$locker_id'" );
+    _log( "unlock request : key '$key_to_unlock' for locker '$locker_id'" );
 
     $self->{_locks}{$key_to_unlock} ||= [];
     my $lockers = $self->{_locks}{$key_to_unlock};
@@ -380,7 +380,7 @@ sub _unlock {
 sub _verify {
     my( $self, $connection, $key_to_check, $locker_id ) = @_;
 
-    _log( "locker server check for key '$key_to_check' for locker '$locker_id'" );
+    _log( "verify : locker server check for key '$key_to_check' for locker '$locker_id'" );
 
     $self->{_locks}{$key_to_check} ||= [];
     my $lockers = $self->{_locks}{$key_to_check};
@@ -388,7 +388,7 @@ sub _verify {
     #check for timed out lockers
     my $t = time;
     while( @$lockers && $t > $self->{_locker_counts}{$lockers->[0]}{$key_to_check} ) {
-        _log( "lock '$key_to_check' timed out for locker '$lockers->[0]'" );
+        _log( "verify:  '$key_to_check' timed out for locker '$lockers->[0]'" );
         if( 1 == keys %{ $self->{_locker_counts}{$lockers->[0]} } ) {
             delete $self->{_locker_counts}{$lockers->[0]};
         } else {
@@ -431,10 +431,10 @@ use IO::Socket::INET;
 =head3 new( lockername, host, port )
 
     Creates a client object with the given name for the host and port.
-    
+
 =cut
 sub new {
-    my( $pkg, $lockerName, $host, $port ) = @_;
+    my( $pkg, $lockerName, $host, $port, $args ) = @_;
     die "Must supply locker name" unless $lockerName;
 
     $host ||= '127.0.0.1';
@@ -445,12 +445,21 @@ sub new {
         host => $host,
         port => $port,
         name => $lockerName,
+        attempts => $args->{reconnect_attemps} || 3,
+        time_between_attempts => $args->{time_between_attempts} || 3,
     }, $class;
 } #new 
 
 sub _get_sock {
     my $self = shift;
-    my $sock = new IO::Socket::INET( "$self->{host}:$self->{port}" );
+    
+    # try a few times, then give up
+    my( $sock, $count );
+    until( $sock || $count++ > $self->{attempts} ) {
+        $sock = new IO::Socket::INET( "$self->{host}:$self->{port}" );
+        sleep $self->{time_between_attempts}*($count) unless $sock;
+    }
+    die "Could not connect : $@" unless $sock;
     binmode $sock, ':utf8';
     $sock;
 }
@@ -537,6 +546,6 @@ __END__
 
 =head1 VERSION
 
-       Version 1.1  (October 12, 2015))
+       Version 1.2  (October 12, 2015))
 
 =cut
